#
# $HeadURL: https://svn.oucs.ox.ac.uk/sysdev/src/packages/r/rb3/trunk/lib/RB3/TemplateFunctions.pm $
# $LastChangedRevision: 26289 $
# $LastChangedDate: 2015-05-13 16:58:10 +0100 (Wed, 13 May 2015) $
# $LastChangedBy: ouit0139 $
#
package RB3::Interface;

=head1 NAME

RB3::Interface - function used to find interfaces in network configs

=head1 DESCRIPTION

The following subroutine is used by the CLI and the template functions

=cut

use strict;
use warnings FATAL => 'all';
use feature 'state';

use File::Spec;
use IO::File;
use Regexp::Common qw( net );
use Socket qw(AF_INET AF_INET6 SOCK_RAW sockaddr_in sockaddr_in6 getaddrinfo
              inet_ntop inet_pton);

sub parse_interfaces {
    my $hostname = shift;
    my $sys_dir = shift;

    # Check if IPv6 regexs are available
    my $v6re_OK = undef;
    eval { $v6re_OK = "$RE{net}{IPv6}"; };

    my ($err, @addrs) = getaddrinfo($hostname, "", {socktype => SOCK_RAW});
        die "Cannot getaddrinfo - $err" if $err;

    my @ipaddr;
    
    foreach my $a (@addrs) {
        if ($a->{family} == AF_INET) {
            # port is always 0 when resolving a hostname
            my ($addr4) = (sockaddr_in($a->{addr}))[1];
            push @ipaddr, inet_ntop(AF_INET, $addr4);
        } else {
            my ($addr6) = (sockaddr_in6($a->{addr}))[1];
            push @ipaddr, inet_ntop(AF_INET6, $addr6);
        }
    }

    my %wanted = map { $_ => 1 } @ipaddr;

    my @hosts;

    foreach my $path ( glob( "$sys_dir/*/root/etc/network/interfaces" ) ) {
        my $fh = IO::File->new( $path, O_RDONLY )
            or die "open $path: $!";
        while ( <$fh> ) {
            my $addr;
            if (
                (
                    ( $addr ) = $_ =~ /^\s+address\s+$RE{net}{IPv4}{-keep}\s*$/ or
                    ( $addr ) = $_ =~ /^\s+(?:pre-|post-)?up\s+ip\s+addr(?:ess)?\s+add\s+$RE{net}{IPv4}{-keep}\/\d+/
                ) or (
                    $v6re_OK and (
                        ( $addr ) = $_ =~ /^\s+address\s+$RE{net}{IPv6}{-keep}\s*$/ or
                        ( $addr ) = $_ =~ /^\s+(?:pre-|post-)?up\s+ip\s+-6\s+addr(?:ess)?\s+add\s+$RE{net}{IPv6}{-keep}\/\d+/
                    )
                )
            ) {
                if ( $wanted{ $addr } ) {
                    ( my $system ) = $path =~ m{^\Q$sys_dir\E/$RE{net}{domain}{-keep}/};
                    push(@hosts, $system);
                    last;
                }
            }
        }
    }
    
    return @hosts;
}

1;

__END__
